package com.jiuqi.sql.impl;

import com.jiuqi.util.Stream;
import com.jiuqi.util.MemStream;
import com.jiuqi.sql.FieldInfo;
import java.io.IOException;
import com.jiuqi.sql.InputRecordBuffer;
import com.jiuqi.sql.DBException;
import com.jiuqi.sql.InputDataSetMetaData;
import java.io.ByteArrayInputStream;
import com.jiuqi.sql.FieldInfoList;
import java.io.InputStream;
import com.jiuqi.sql.InputDataSet;

public class DBaseInputDataSet extends InputDataSet
{
    private InputStream stream;
    private DBaseFileHeader header;
    private DBaseFileTail tail;
    private DBaseInputRecordBuffer buffer;
    private FieldInfoList infoList;
    
    private int getFieldType(final char dataType) {
        int result = dataType;
        switch (dataType) {
            case 'C':
            case 'F':
            case 'W': {
                result = 6;
                break;
            }
            case 'L': {
                result = 1;
                break;
            }
            case '+': {
                result = 7;
                break;
            }
            case '%':
            case 'O': {
                result = 3;
                break;
            }
            case 'A':
            case 'I':
            case 'R': {
                result = 5;
                break;
            }
            case '@':
            case 'D': {
                result = 2;
                break;
            }
            case 'Z': {
                result = 8;
                break;
            }
        }
        return result;
    }
    
    public DBaseInputDataSet(final InputStream inStream) {
        this.header = new DBaseFileHeader();
        this.tail = new DBaseFileTail();
        this.stream = inStream;
    }
    
    public DBaseInputDataSet(final byte[] data) {
        this(new ByteArrayInputStream(data));
    }
    
    public FieldInfoList getFields() {
        return this.infoList;
    }
    
    public InputDataSetMetaData getMetaData() throws DBException {
        return (InputDataSetMetaData)new DBaseInputDataSetMetaData(this.infoList);
    }
    
    protected boolean getNextRecord(final InputRecordBuffer buffer) throws DBException {
        return buffer.readRecord(this.stream);
    }
    
    protected void initFields(final FieldInfoList fieldInfos) throws DBException {
        fieldInfos.clear();
        try {
            this.header.readFromStream(this.stream);
            final boolean supportText = this.header.ver == -115;
            final int fieldCount = (this.header.structDefLen - 68 - 1) / 50;
            if (fieldCount == 0) {
                throw new DBException("Not Field Define！");
            }
            final DBaseFieldHeader fieldHeader = new DBaseFieldHeader();
            int fieldOffset = 0;
            for (int i = 0; i < fieldCount; ++i) {
                fieldHeader.readFromStream(this.stream);
                final FieldInfo fieldInfo = fieldInfos.add();
                fieldInfo.name = fieldHeader.fieldName;
                fieldInfo.incValue = fieldHeader.incValue;
                fieldInfo.offset = fieldOffset;
                fieldInfo.size = fieldHeader.fieldLength;
                fieldInfo.decimal = fieldHeader.scale;
                fieldInfo.type = this.getFieldType(fieldHeader.dataType);
                if (supportText && fieldInfo.type == 6) {
                    fieldOffset += 4;
                }
                else {
                    fieldOffset += fieldHeader.fieldLength;
                }
            }
            if (fieldOffset != this.header.recLength - 1) {
                throw new DBException("Field Define Infomation Error");
            }
            this.stream.read();
            this.stream.mark(0);
            this.stream.skip(this.header.recCount * this.header.recLength + 1L);
            this.tail.readFromStream(this.stream);
            this.buffer = new DBaseInputRecordBuffer(this.header.recLength - 1, supportText, this.tail);
            this.infoList = fieldInfos;
            this.stream.reset();
        }
        catch (final IOException ex) {
            final DBException dbe = new DBException();
            dbe.setStackTrace(ex.getStackTrace());
            throw dbe;
        }
    }
    
    protected InputRecordBuffer internalOpen() throws DBException {
        return (InputRecordBuffer)this.buffer;
    }
    
    protected void internalClose() throws DBException {
        this.buffer = null;
    }
    
    public void first() throws DBException {
        try {
            this.stream.reset();
        }
        catch (final Exception ex) {
            throw new DBException((Throwable)ex);
        }
    }
    
    public static void main(final String[] args) throws Exception {
        final Stream s = (Stream)new MemStream();
        s.loadFromFile("d:\\1.dbf");
        final InputStream as = new ByteArrayInputStream(s.getBytes());
        final InputDataSet dataset = new DBaseInputDataSet(as);
        dataset.open();
        while (dataset.next()) {
            System.out.print(dataset.getBoolean(1));
            if (dataset.wasNull()) {
                System.out.print("[ISNULL]");
            }
            System.out.print("\t");
            System.out.print(dataset.getDouble(2));
            if (dataset.wasNull()) {
                System.out.print("[ISNULL]");
            }
            System.out.print("\t");
            System.out.print(dataset.getInt(3));
            if (dataset.wasNull()) {
                System.out.print("[ISNULL]");
            }
            System.out.print("\t");
            System.out.print(dataset.getString(4));
            if (dataset.wasNull()) {
                System.out.print("[ISNULL]");
            }
            System.out.print("\t");
            System.out.print(dataset.getLong(5));
            if (dataset.wasNull()) {
                System.out.print("[ISNULL]");
            }
            System.out.println("\t");
        }
        System.out.println("OK");
    }
}
